
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// DownloadDialog.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "DownloadDialog.h"
#include <afxdlgs.h>
#include "shlobj.h" // For SHFunctions
#include "VideoStoreDlg.h"

#define LINESIZE 8

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDownloadDialog dialog


CDownloadDialog::CDownloadDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CDownloadDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDownloadDialog)
	m_strEditMsg = _T("");
	m_strFolderPath = _T("");
	//}}AFX_DATA_INIT

	// Scrolling
	m_nDlgHeight	= 0;	// Workspace height in pixels
	m_nVScrollPos	= 0;	// Vertical scroll position
	m_nVPageSize	= 0;	// Vertical page size
}

CDownloadDialog::~CDownloadDialog()
{
	((CApp *)AfxGetApp())->m_hwndDownloadDlg = NULL;
}


void CDownloadDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDownloadDialog)
	DDX_Control(pDX, IDC_BUTTON_CHANGE_PATH, m_ctrlButtonFolder);
	DDX_Control(pDX, IDC_BUTTON_CLEAR, m_ctrlButtonClear);
	DDX_Control(pDX, IDC_BUTTON_DOWNLOAD, m_ctrlDownloadButton);
	DDX_Control(pDX, IDC_DOWNLOAD_PROG_VID, m_ctrlProgressVid);
	DDX_Control(pDX, IDC_DOWNLOAD_PROG, m_ctrlProgress);
	DDX_Control(pDX, IDC_DOWNLOAD_MSG, m_ctrlEditMsg);
	DDX_Control(pDX, IDC_STATIC_NUM_VIDS, m_ctrlStaticNumVids);
	DDX_Control(pDX, IDC_STATIC_KBPS, m_ctrlStaticKBps);
	DDX_Control(pDX, IDC_LIST_BOX, m_ctrlListBox);
	DDX_Text(pDX, IDC_DOWNLOAD_MSG, m_strEditMsg);
	DDX_Text(pDX, IDC_EDIT_FOLDER_PATH, m_strFolderPath);
	//}}AFX_DATA_MAP

}


BEGIN_MESSAGE_MAP(CDownloadDialog, CDialog)
	//{{AFX_MSG_MAP(CDownloadDialog)
	ON_BN_CLICKED(IDC_BUTTON_DOWNLOAD, OnButtonDownload)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_PATH, OnChangeFolderPath)
	ON_WM_VSCROLL()
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, OnButtonClear)
	//}}AFX_MSG_MAP

	ON_MESSAGE (WM_USER_STATUS_STRING,	OnStatusString)
	ON_MESSAGE (WM_USER_PROGRESS_POS,	OnProgressPos)
	ON_MESSAGE (WM_USER_PROGRESS_MAX,	OnProgressMax)
	ON_MESSAGE (WM_USER_PROGRESS2_POS,	OnProgress2Pos)
	ON_MESSAGE (WM_USER_PROGRESS2_MAX,	OnProgress2Max)
	ON_MESSAGE (WM_USER_FINISHED,		OnDownloadFinished)
	ON_MESSAGE (WM_USER_DLG_UPDATE,		OnUpdateList)
	ON_MESSAGE (WM_USER_KBPS,			OnKBps)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDownloadDialog message handlers

BOOL CDownloadDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here

	// Set CApp handle.
	((CApp *)AfxGetApp())->m_hwndDownloadDlg = m_hWnd;

	// Set directory per CDoc user settings.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(pDoc)
		m_strFolderPath = pDoc->m_videoStorePath;
	else
		m_strFolderPath = _T("C:\\Khan_video_folder");

	// Check if this is a valid folder.
	CVideoStoreDlg dlg;
	CString errMsg;
	int ret = dlg.ValidateFolder(m_strFolderPath, FALSE, errMsg);

	// Invalid folder path.
	// Can occure with removable dives.
	if(ret == 0)
	{
		m_strEditMsg.Format(_T("Invalid Video Storage folder.\r\n%s\r\n")
							_T("Please select a new folder by\r\n")
							_T("pressing the Change button"), errMsg);
		m_ctrlDownloadButton.EnableWindow(FALSE);

	}
	// New folder. Let user know it will be created.
	else if(ret == 2)
	{
		m_strEditMsg.Format(_T("%s\r\n")
							_T("will be created automatically. Press\r\n")
							_T("Press the Change button to\r\n")
							_T("select another folder"), m_strFolderPath);
	}

	// Set initial message pane text.
	//m_strEditMsg = _T("Press the download button to begin.");

	// Set list.
	m_ctrlListBox.InitStorage(25, 10);

	// Set KBps string.
	m_ctrlStaticKBps.SetWindowText(_T("Download Rate (KBps) :  0.0"));

	OnUpdateList(0, 0);

	// Set controls.
	UpdateData(FALSE);

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Download the videos.
void CDownloadDialog::OnButtonDownload() 
{
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// not editable
	// Get any user directory edits.
	//UpdateData(TRUE); 

	// Get current Button text.
	CString str;
	m_ctrlDownloadButton.GetWindowText(str);

	// Download via CDoc.
	if(str == _T("Download"))
	{
		// If none, bail.
		if(!pDoc->NumVideosForDownload())
			return;

		// Get storage path.
		CString path = pDoc->m_videoStorePath;

		// Create directory as needed.
		if(!CreateDirectory(path, NULL))
		{
			DWORD err = GetLastError();

			// If exists.
			if(err == ERROR_ALREADY_EXISTS) // code 183
			{
				// Ok - continue.
			}

			// Cannot be created. Bail.
			// Note also checked in OnInitDialog() at start up.
			// See also CVideoStoreDlg.
			else
			{
				// Format last error.
				LPVOID lpMsgBuf;
				FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
							  FORMAT_MESSAGE_FROM_SYSTEM	 |
							  FORMAT_MESSAGE_IGNORE_INSERTS,
							  NULL,
							  err,
							  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
							  (LPTSTR) &lpMsgBuf,
							  0,
							  NULL);

				// Format err msg.
				m_strEditMsg.Format(_T("Invalid Video Storage folder.\r\n%s\r\n")
									_T("Please select a new folder by\r\n")
									_T("pressing the Change button"), lpMsgBuf);
				m_ctrlDownloadButton.EnableWindow(FALSE);

				// Clean up.
				LocalFree(lpMsgBuf);

				// Set controls.
				UpdateData(FALSE);

				return;		
			}
		}

		// If we can download. 
		if(pDoc->DownloadVideos( GetSafeHwnd() ))
		{
			// Set button text.
			m_ctrlDownloadButton.SetWindowText(_T("Cancel"));
			// Disable.
			m_ctrlButtonClear.EnableWindow(FALSE);
			m_ctrlButtonFolder.EnableWindow(FALSE);
		}

	}
	// Cancel via CDoc.
	else if(str == _T("Cancel"))
	{
		pDoc->CancelVideoDownload();
	}
}

// Called from TreeDlg when video marked for download.
LRESULT CDownloadDialog::OnUpdateList(WPARAM wParam, LPARAM lParam)	// WM_USER_DLG_UPDATE
{
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return 0;

	// Clean.
	m_ctrlListBox.ResetContent();

	// Get vids for download.
	CString str;
	CVideo video;
	int sz = pDoc->NumVideosForDownload();
	for(int i = 0; i < sz; i++)
	{
		// Get valid video file name.
		if(!pDoc->GetVideoForDownload(i, video))
			return 0;
		// Add title to list. Note file name is useless as of v0.8.4.
		m_ctrlListBox.AddString(video.title);

	/*pre v 0.8.4	
	// Get valid video file name.
		if(!pDoc->GetVideoForDownload(i, video))
			return 0;
		if(!video.GetShortFileName(str))
			return 0;
		// Add to list
		m_ctrlListBox.AddString(str);*/
	}

	// Set text.
	CString s;
	s.Format(_T("%d videos selected for download."), sz);
	m_ctrlStaticNumVids.SetWindowText(s);

	// Set horiz extent
	if(sz)
	{
		// Find the longest string in the list box.
		CString str;
		CSize   sz;
		int     dx = 0;
		CDC *   pDC = m_ctrlListBox.GetDC();
		CFont * pFont = GetFont();					// Dialog font.
		CFont * old = pDC->SelectObject(pFont);		// Must select font.
		int nItems = m_ctrlListBox.GetCount();
		for(int i = 0; i < nItems; i++)
		{
			m_ctrlListBox.GetText(i, str);
			sz = pDC->GetTextExtent(str);

			if(sz.cx > dx)
				dx = sz.cx;
		}
		pDC->SelectObject(old);
		m_ctrlListBox.ReleaseDC(pDC);

		// Set the horizontal extent so every character of 
		// all strings can be scrolled to.
		m_ctrlListBox.SetHorizontalExtent(dx + 5);	// Add a few pix.
	}
	return 0;
}


// Called from Worker Thread via SendMessage.
LRESULT CDownloadDialog::OnStatusString(WPARAM wParam, LPARAM lParam)	// WM_USER_STATUS_STRING
{
	// Get message string from CApp.
	CString str;
	if(	((CApp*)AfxGetApp())->GetThreadMessage(str) )	// move to CMainFrame?
	{
		TRACE(_T("CDownloadDialog::OnStatusString\n"));

		// Set the CEdit
		m_ctrlEditMsg.SetWindowText(str);
		m_ctrlEditMsg.LineScroll(m_ctrlEditMsg.GetLineCount());
		m_ctrlEditMsg.UpdateWindow();
	}
    return 0;
}


// Called with Dialog Handle from worker thread.
//
// Total progress pos.
LRESULT CDownloadDialog::OnProgressPos(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS_POS
{
	TRACE(_T("CDownloadDialog OnProgressPos Pos = %d\n"), (int)lParam);

	m_ctrlProgress.SetPos((int)lParam);
    return 0;
}
// Total progress range.
LRESULT CDownloadDialog::OnProgressMax(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS_MAX
{
	TRACE(_T("CDownloadDialog OnProgressMax Max = %d\n"), (int)lParam);

	m_ctrlProgress.SetRange32(0, (int)lParam);
    return 0;
}
// Video progress pos.
LRESULT CDownloadDialog::OnProgress2Pos(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS2_POS
{
	TRACE(_T("CDownloadDialog OnProgress2Pos Pos = %d\n"), (int)lParam);

	m_ctrlProgressVid.SetPos((int)lParam);
    return 0;
}
// Video progress range.
LRESULT CDownloadDialog::OnProgress2Max(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS2_MAX
{
	TRACE(_T("CDownloadDialog OnProgress2Max Max = %d\n"), (int)lParam);

	m_ctrlProgressVid.SetRange32(0, (int)lParam);
    return 0;
}

// KB per second.
// wparam is the seconds, lparam is a fraction
LRESULT CDownloadDialog::OnKBps(WPARAM wParam, LPARAM lParam)		// WM_USER_KBPS
{
	double sec = (double)(int)wParam;
	double fract = (double)(int)lParam;
	fract /= 10;
	sec += fract;

	// Set static member str.
	CString s;
	s.Format(_T("Download Rate (KBps) :  %.1f"), sec);
	m_ctrlStaticKBps.SetWindowText(s);

	TRACE(_T("KBps: %.1f\n"), sec);
    return 0;
}

// Finished download. Reset ctrls.
LRESULT CDownloadDialog::OnDownloadFinished(WPARAM wParam, LPARAM lParam)	// WM_USER_FINISHED
{
	// Reset button text.
	m_ctrlDownloadButton.SetWindowText(_T("Download"));
	// Enable Buttons.
	m_ctrlButtonClear.EnableWindow(TRUE);
	m_ctrlButtonFolder.EnableWindow(TRUE);

	// Reset progress ctrls.
	m_ctrlProgress.SetPos(0);
	m_ctrlProgressVid.SetPos(0);

	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return 0;

	// Update the m_ctrlListBox list box.
	OnUpdateList(0, 0);

	// Zero KBps string.
	m_ctrlStaticKBps.SetWindowText(_T("Download Rate (KBps) :  0.0"));

	// Note leave the m_strEditMsg in place till next 
	// download.

	// Update the Contents Tree.

	return 0;
}

// Change the video storage folder.
void CDownloadDialog::OnChangeFolderPath() 
{
	CVideoStoreDlg dlg;
	if(dlg.DoModal() == IDOK)
	{
		// CVideoStoreDlg will have validated, and
		// if needed created, the storage folder.
		m_ctrlDownloadButton.EnableWindow(TRUE);

		// Set CDoc.
		CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
		if(pDoc)
			pDoc->m_videoStorePath = dlg.m_strFolderPath;

		// Set local.
		m_strFolderPath = dlg.m_strFolderPath;

		// Clear text.
		m_strEditMsg.Empty();

		// Update windows.
		UpdateData(FALSE);
	}	
}

void CDownloadDialog::OnVScroll(UINT nCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	
//	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);

    int nDelta;

    // Compute the vertical scroll distance, or "delta."
    switch (nCode) 
	{

    case SB_LINEUP:
        nDelta = -LINESIZE;
        break;

    case SB_PAGEUP:
        nDelta = -m_nVPageSize;
        break;

    case SB_THUMBTRACK:
        nDelta = (int) nPos - m_nVScrollPos;
        break;

    case SB_PAGEDOWN:
        nDelta = m_nVPageSize;
        break;

    case SB_LINEDOWN:
        nDelta = LINESIZE;
        break;

    default: // Ignore other scroll bar messages
        return;
    }

    // Adjust the delta if adding it to the current scroll position would
    // cause an underrun or overrun.
    int nScrollPos = m_nVScrollPos + nDelta;
    int nMaxPos = m_nDlgHeight - m_nVPageSize;

    if (nScrollPos < 0)
        nDelta = -m_nVScrollPos;
    else if (nScrollPos > nMaxPos)
        nDelta = nMaxPos - m_nVScrollPos;

    // Update the scroll position and scroll the window.
    if (nDelta != 0) 
	{
        m_nVScrollPos += nDelta;
        SetScrollPos (SB_VERT, m_nVScrollPos, TRUE);
        ScrollWindow(0, -nDelta);
    }
	CDialog::OnVScroll(nCode, nPos, pScrollBar);
}

void CDownloadDialog::OnSize(UINT nType, int cx, int cy) 
{
	TRACE(_T("In CDownloadDialog OnSize\n"));
	int nVScrollMax = 0;

	// Get dialog height from app - initialized in CTabView::CreatePages.
	m_nDlgHeight = ((CApp*)AfxGetApp())->m_downloadDlgRect.Height();

	// Initialize and update our scrollinfo for vert bar.
	//
	// If dialog is larger than window, adjust bars
	if (cy < m_nDlgHeight) 
	{
		nVScrollMax = m_nDlgHeight - 1;
		m_nVPageSize = cy;
	}

	// Else, effectively hides bars
	else
	{
		// no scroll bars are needed. Position the dlg back to normal
		ScrollWindow(0, m_nVScrollPos);
		// init scrollinfo
		m_nVScrollPos = m_nVPageSize = 0;
	}

	SCROLLINFO si;
	si.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
	si.nMin = 0;
	si.nMax = nVScrollMax;
	si.nPos = m_nVScrollPos;
	si.nPage = m_nVPageSize;

	SetScrollInfo (SB_VERT, &si, TRUE);

	// dlg does not repaint after update. This forces the issue.
	Invalidate();
	if(si.nPos != 0)
		PostMessage(WM_VSCROLL);

	CDialog::OnSize(nType, cx, cy);	
	
}

// Remove all Videos to be downlaoded.
void CDownloadDialog::OnButtonClear() 
{
	// TODO: Add your control notification handler code here
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// Pdoc will empty the CDoc vid download array,
	// call the tree dlg to update, and call this
	// dlg's OnDownloadFinished().
	pDoc->RemoveAllVideosForDownload();	
}

// Ignore Return, Escape and Space keys.
BOOL CDownloadDialog::PreTranslateMessage(MSG* pMsg) 
{
	// TODO: Add your specialized code here and/or call the base class	

	if(pMsg->message == WM_KEYDOWN		&&
	   ( pMsg->wParam == VK_RETURN		|| 
		 pMsg->wParam == VK_ESCAPE		||
		 pMsg->wParam == VK_SPACE )		)
	{
		return TRUE;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}
