// MsiMod.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "MsiMod.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Settings:
// Project Settings/Link/ settings - dbg and release
// add msi.lib 

//
// Note: Be sure to copy/paste a new "KhanSetup.msi" from
//		 the "Copy of KhanSetup.msi" before running in 
//		 debug. For release, param must be supplied.
//

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	// Initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return 1;
	}

	// Begin.
	cout << endl;
	cout << _T("Begin MsiMod - Validating arguments.") << endl;

	// Need 2 args. 
	// Note arg for debug set in Settings/Debug/Program arguments. See below.
	if(argc != 2)
	{
		cout << _T("Incorrect arguments.") << endl;
		return 0;
	}

	// Print command lines.
	CString str;
	for(int i = 0; i < argc; i++)
	{
		str.Format(_T("Arg # %d"), i);
		cout << (LPCTSTR)str << endl;
		str = argv[i];
		cout << (LPCTSTR)str << endl;
	}

	// Paths.
	CString command = argv[0];			// run command for this app
	CString msiPath = argv[1];			// msi path - use quotes if spaces.

//	vs2005 dbg command line msiPath arg = 
//		"D:\Programming(D)\Workspaces\VS_2005 Workspaces\MFC\Testing\MsiMod\KhanSetup.msi"
//		"$(ProjectDir)KhanSetup.msi"

	PMSIHANDLE  database = NULL;
	UINT ret = 0;

	// Open msi.
	ret = MsiOpenDatabase(msiPath, 
						  MSIDBOPEN_TRANSACT, // MSIDBOPEN_TRANSACT MSIDBOPEN_DIRECT
						  &database); 
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiOpenDatabase"));

	// Set permissions.
	cout << endl;
	ret = SetPermissions(database);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("SetPermissions"));
	else
		cout << (LPCTSTR)_T("SetPermissions successfull!") << endl;

	// Set conditional shortcuts.
	cout << endl;
	ret = SetConditionalShortcuts(database);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("SetConditionalShortcuts"));
	else
		cout << (LPCTSTR)_T("SetConditionalShortcuts successfull!") << endl;

	return 0;
}

// Set permissions for KhanDB.dat file and COMMONAPPDATAFOLDER directory.
int SetPermissions(PMSIHANDLE & database)
{
	if(database == NULL)
	{
		cout << (LPCTSTR)_T("No database handle for SetConditionalShortcuts.") << endl;
		return 0;
	}

	/////////////////////////////////////////////
	// Existing strings in msi - modify per msi:
	CString KhanDbFileName = _T("KHANDB.DAT|KhanDB.dat");
	CString CommonAppData  = _T("COMMONAPPDATAFOLDER");

	////////////////
	// New strings.
	CString Everyone	= _T("Everyone");
	int		GenericAll	= 268435456;			// Permission GENERIC_ALL

	/////////
	// Vars.
	PMSIHANDLE  view = NULL;
	PMSIHANDLE  record = NULL;
	UINT ret = 0;
	CString query, FileKey, DirKey, LockObject, Table, Domain, User, msg;
	int Permission;

	////////////////////////////////////////
	// Set permissions for KhanDb.dat file.
	//
	// Get KhanDb.dat File field from File Table.
	query.Format(_T("SELECT File.File FROM File WHERE FileName = '%s'"),
						KhanDbFileName);	

	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView file field"));

	// Execute
	ret = MsiViewExecute(view, record );
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute file field"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch file field"));

	// Get file key string.
	if(!GetRecordString(record, 1, FileKey))
		return Error(ret, _T("GetRecordString file field"));

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose file field"));

	msg.Format(_T("KhanDB.dat File Key: %s"), FileKey);
	cout << (LPCTSTR)msg << endl;

	// Insert LockPermissions record.
	LockObject	= FileKey;
	Table		= _T("File");
	Domain		= _T("");		// empty
	User		= Everyone; 
	Permission	= GenericAll;

	ret = MsiDatabaseOpenView(database, _T("SELECT * FROM LockPermissions"), &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView permissions KhanDB"));

	record = MsiCreateRecord(5);
	if(!SetRecordString(record, 1, LockObject))
		return Error(ret, _T("SetRecordString permissions KhanDB 1"));
	if(!SetRecordString(record, 2, Table))
		return Error(ret, _T("SetRecordString permissions KhanDB 2"));
	if(!SetRecordString(record, 3, Domain))
		return Error(ret, _T("SetRecordString permissions KhanDB 3"));
	if(!SetRecordString(record, 4, Everyone))
		return Error(ret, _T("SetRecordInt permissions KhanDB 4"));
	if(!SetRecordInt(record, 5, GenericAll))
		return Error(ret, _T("SetRecordString permissions KhanDB 5"));

	ret = MsiViewModify(view, MSIMODIFY_INSERT, record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewModify permissions KhanDB"));
	// Ok
	cout << (LPCTSTR)_T("Permissions set for KhanDB file.") << endl;

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose permissions KhanDB"));


	//////////////////////////////////////////////////////
	// Set permissions for COMMONAPPDATAFOLDER directory. 
	//
	// Get Directory field from Directory Table.
	query.Format(_T("SELECT Directory.Directory FROM Directory ")
				 _T("WHERE Directory_Parent = '%s'"),
					CommonAppData);	

	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView directory field"));

	// Execute
	ret = MsiViewExecute(view, record );
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute directory field"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch directory field"));

	// Get file key string.
	if(!GetRecordString(record, 1, DirKey))
		return Error(ret, _T("GetRecordString directory field"));// 

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose directory field"));

	msg.Format(_T("CommonAppData directory Key: %s"), DirKey);
	cout << (LPCTSTR)msg << endl;

	// Insert LockPermissions record.
	LockObject	= DirKey;
	Table		= _T("Directory");
	Domain		= _T("");		// empty
	User		= Everyone; 
	Permission	= GenericAll;

	ret = MsiDatabaseOpenView(database, _T("SELECT * FROM LockPermissions"), &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView permissions KhanDB"));

	record = MsiCreateRecord(5);
	if(!SetRecordString(record, 1, LockObject))
		return Error(ret, _T("SetRecordString permissions directory 1"));
	if(!SetRecordString(record, 2, Table))
		return Error(ret, _T("SetRecordString permissions directory 2"));
	if(!SetRecordString(record, 3, Domain))
		return Error(ret, _T("SetRecordString permissions directory 3"));
	if(!SetRecordString(record, 4, Everyone))
		return Error(ret, _T("SetRecordInt permissions directory 4"));
	if(!SetRecordInt(record, 5, GenericAll))
		return Error(ret, _T("SetRecordString permissions directory 5"));

	ret = MsiViewModify(view, MSIMODIFY_INSERT, record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewModify permissions directory"));

	// Ok
	cout << (LPCTSTR)_T("Permissions set for COMMONAPPDATAFOLDER directory.") << endl;

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose permissions directory"));

	// Commit changes.
	ret = MsiDatabaseCommit(database);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseCommit SetPermissions"));

	cout << endl;
	cout << (LPCTSTR)_T("SetPermissions - Done.") << endl;

	return 0;
}


// Setup conditional shortcuts for Start Menu and Desktop
// per two checkboxes.
int SetConditionalShortcuts(PMSIHANDLE & database)
{
	if(database == NULL)
	{
		cout << (LPCTSTR)_T("No database handle for SetConditionalShortcuts.") << endl;
		return 0;
	}

	/////////////////////////////////////////////
	// Existing strings in msi - modify per msi:
	//
	// Exe File.FileName - Khan Desktop.
	CString ExeFileName = _T("KHAN.EXE|Khan.exe");

	// Desktop shortcut condition.
	CString DesktopCondition = _T("CHECKBOXA1");

	// Start Menu shortcut condition.
	CString StartMenuCondition = _T("CHECKBOXA2");

	// Shortcut Table Directory_ string for desktop shortcut.
	CString DesktopDir = _T("DesktopFolder");

	// Shortcut Table Directory_ string for program menu shortcut.
	CString StartMenuDir = _T("ProgramMenuFolder");

	///////////////////////
	// New strings we set.
	CString DesktopComponent	 = _T("COMPONENT_DESKTOP");
	CString DesktopComponentId	 = _T("{924C04F0-FEFD-4B19-9F4A-4D1E53F4929F}");// it's unique
	CString StartMenuComponent	 = _T("COMPONENT_STARTMENU");
	CString StartMenuComponentId = _T("{E4B06EC6-D96A-40C3-9FAB-A8E5482B2635}");

	/////////
	// Vars.
	PMSIHANDLE  view = NULL;
	PMSIHANDLE  record = NULL;
	UINT ret = 0;
	int Attributes; 
	CString query, Component, ComponentId, Directory_, Condition, KeyPath, msg;

	/////////////////////////////////////////////
	// Get exe Component_ field from File Table.
	//
	query.Format(_T("SELECT Component_ FROM File WHERE File.FileName = '%s'"),
						ExeFileName);	

	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView"));

	// Execute
	ret = MsiViewExecute(view, record );
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute exe"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch exe"));

	// Get exe component string.
	if(!GetRecordString(record, 1, Component))
		return Error(ret, _T("GetRecordString exe"));

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose exe"));

	msg.Format(_T("Exe component: %s"), Component);
	cout << (LPCTSTR)msg << endl;

	////////////////////////////////
	// Get exe Componets Table row.
	//
	query.Format(_T("SELECT ComponentId, Directory_, Attributes, Condition, KeyPath ")
				 _T("FROM Component WHERE Component.Component = '%s'"), Component);

	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView exe comp"));

	// Execute
	ret = MsiViewExecute(view, record); // ERROR_FUNCTION_FAILED  ERROR_INVALID_HANDLE  
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute exe comp"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch exe comp"));

	// Get exe ComponentId string.
	if(!GetRecordString(record, 1, ComponentId))
		return Error(ret, _T("GetRecordString exe comp 1"));
	// Get Directory_ string.
	if(!GetRecordString(record, 2, Directory_))
		return Error(ret, _T("GetRecordString exe comp 2"));
	// Get Attributes int.
	if(!GetRecordInt(record, 3, Attributes))
		return Error(ret, _T("GetRecordInt exe comp 3"));
	// Get Condition string.
	if(!GetRecordString(record, 4, Condition))
		return Error(ret, _T("GetRecordString exe comp 4"));
	// Get KeyPath string.
	if(!GetRecordString(record, 5, KeyPath))
		return Error(ret, _T("GetRecordString exe comp 5"));

	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose exe comp"));

	/////////////////////////////////////////////////////////////
	// Insert Desktop shortcut component, based on exe component.
	//
	Component	= DesktopComponent;
	ComponentId = DesktopComponentId;
	Condition	= DesktopCondition;

	ret = MsiDatabaseOpenView(database, _T("SELECT * FROM `Component`"), &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose insert desktop"));
	record = MsiCreateRecord(6);
	if(!SetRecordString(record, 1, Component))
		return Error(ret, _T("SetRecordString insert desktop 1"));
	if(!SetRecordString(record, 2, ComponentId))
		return Error(ret, _T("SetRecordString insert desktop 2"));
	if(!SetRecordString(record, 3, Directory_))
		return Error(ret, _T("SetRecordString insert desktop 3"));
	if(!SetRecordInt(record, 4, Attributes))
		return Error(ret, _T("SetRecordInt insert desktop 4"));
	if(!SetRecordString(record, 5, Condition))
		return Error(ret, _T("SetRecordString insert desktop 5"));
	if(!SetRecordString(record, 6, KeyPath))
		return Error(ret, _T("SetRecordString insert desktop 6"));	
	ret = MsiViewModify(view, MSIMODIFY_INSERT, record);
	if(ret != ERROR_SUCCESS)
		cout << (LPCTSTR)_T("MsiViewModify failed - ")
						 _T("Desktop Shortcut component previously set?") << endl;
	else
		cout << (LPCTSTR)_T("Desktop Shortcut Component set.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose insert desktop"));

	/////////////////////////////////////////////////////////////////
	// Insert Start Menu shortcut component, based on exe component.
	//
	Component	= StartMenuComponent;
	ComponentId = StartMenuComponentId;
	Condition	= StartMenuCondition;


	ret = MsiDatabaseOpenView(database, _T("SELECT * FROM `Component`"), &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView insert start"));
	record = MsiCreateRecord(6);
	if(!SetRecordString(record, 1, Component))
		return Error(ret, _T("SetRecordString insert start 1"));
	if(!SetRecordString(record, 2, ComponentId))
		return Error(ret, _T("SetRecordString insert start 2"));
	if(!SetRecordString(record, 3, Directory_))
		return Error(ret, _T("SetRecordString insert start 3"));
	if(!SetRecordInt(record, 4, Attributes))
		return Error(ret, _T("SetRecordInt insert start 4"));
	if(!SetRecordString(record, 5, Condition))
		return Error(ret, _T("SetRecordString insert start 5"));
	if(!SetRecordString(record, 6, KeyPath))
		return Error(ret, _T("SetRecordString insert start 6"));	
	ret = MsiViewModify(view, MSIMODIFY_INSERT, record);
	if(ret != ERROR_SUCCESS)
		cout << (LPCTSTR)_T("MsiViewModify failed - ")
						 _T("Start Menu Shortcut component previously set?") << endl;
	else
		cout << (LPCTSTR)_T("Start Menu Shortcut Component set.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose insert start"));

	//////////////////////////////////////////////
	// Add FeatureComponent for Desktop shortcut.
	//
	query.Format(_T("INSERT INTO `FeatureComponents` ")
				 _T("(FeatureComponents.Feature_, FeatureComponents.Component_) ")
				 _T("VALUES ('%s', '%s')"), 
					_T("DefaultFeature"), DesktopComponent );
	// Open view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView feature desktop"));
	// Execute
	ret = MsiViewExecute(view, record); 
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute feature desktop"));
	cout << (LPCTSTR)_T("Desktop Shortcut FeatureComponent set.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose feature desktop"));

	/////////////////////////////////////////////////
	// Add FeatureComponent for Start Menu shortcut.
	//
	query.Format(_T("INSERT INTO `FeatureComponents` ")
				 _T("(FeatureComponents.Feature_, FeatureComponents.Component_) ")
				 _T("VALUES ('%s', '%s')"), 
					_T("DefaultFeature"), StartMenuComponent);
	// Open view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView feature start"));
	// Execute
	ret = MsiViewExecute(view, record); 
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute StartMenu FeatureComponent"));
	cout << (LPCTSTR)_T("Start Menu Shortcut FeatureComponent set.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose feature start"));

	///////////////////////////////////////////////
	// Modify Shortcut table for Desktop shortcut.
	//
	query.Format(_T("SELECT Shortcut.Shortcut, Directory_, Name, Component_ ")
				 _T("FROM Shortcut WHERE Shortcut.Directory_ = '%s'"), 
						DesktopDir);
	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView mod desktop"));

	// Execute
	ret = MsiViewExecute(view, record);  
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute mod desktop"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch mod desktop"));

	// Update component string.
	if(!SetRecordString(record, 4, DesktopComponent) )
		return Error(ret, _T("SetRecordString mod desktop"));

	ret = MsiViewModify(view, MSIMODIFY_UPDATE, record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewModify desktop shortcut"));
	else
		cout << (LPCTSTR)_T("Desktop shortcut modified.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose mod desktop"));

	//////////////////////////////////////////////////
	// Modify Shortcut table for Start Menu shortcut.
	//
	query.Format(_T("SELECT Shortcut.Shortcut, Directory_, Name, Component_ ")
				 _T("FROM Shortcut WHERE Shortcut.Directory_ = '%s'"), 
						StartMenuDir);
	// Open a view.
	ret = MsiDatabaseOpenView(database, query, &view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseOpenView mod start"));

	// Execute
	ret = MsiViewExecute(view, record);  
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewExecute mod start"));

	// Fetch
	ret = MsiViewFetch(view, &record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewFetch mod start"));

	// Update component string.
	if(!SetRecordString(record, 4, StartMenuComponent) )
		return Error(ret, _T("SetRecordString mod start"));

	ret = MsiViewModify(view, MSIMODIFY_UPDATE, record);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewModify start menu shortcut"));
	else
		cout << (LPCTSTR)_T("Start menu shortcut modified.") << endl;
	// Ensure
	ret = MsiViewClose(view);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiViewClose mod start"));

	// Commit changes.
	ret = MsiDatabaseCommit(database);
	if(ret != ERROR_SUCCESS)
		return Error(ret, _T("MsiDatabaseCommit SetConditionalShortcuts"));

	cout << endl;
	cout << (LPCTSTR)_T("SetConditionalShortcuts - Done.") << endl;
	return 0;
}

///////////////////////////////////////////
// Helpers

// Get a records string.
BOOL GetRecordString(PMSIHANDLE & record, int field, CString & str)
{
	TCHAR buff[256];
	DWORD sz = sizeof(buff) / sizeof(TCHAR);
	ZeroMemory(buff, sz);

	int ret = MsiRecordGetString(record, field, buff, &sz);
	if(ret != ERROR_SUCCESS)
		return FALSE;
	str = buff;

	return TRUE;
}
// Get a records int.
BOOL GetRecordInt(PMSIHANDLE & record, int field, int & intVal)
{
	intVal = MsiRecordGetInteger(record, field);
	if(intVal == MSI_NULL_INTEGER)
		return FALSE;
	else
		return TRUE;
}

// Set a records string.
BOOL SetRecordString(PMSIHANDLE & record, int field, CString & str)
{
	UINT ret = MsiRecordSetString(record, field, str);
	if(ret != ERROR_SUCCESS)
		return FALSE;
	return TRUE;
}
// Set a records int.
BOOL SetRecordInt(PMSIHANDLE & record, int field, int & intVal)
{
	UINT ret = MsiRecordSetInteger(record, field, intVal);
	if(ret != ERROR_SUCCESS)
		return FALSE;
	return TRUE;
}

// Write an error string.
int Error(UINT retCode, CString description)
{
	CString s;
	s.Format(_T("%s failed with code %d."), description, retCode);
	cout << (LPCTSTR)s << endl;
	return 1;
}

// not used
BOOL IsViewModifyError(PMSIHANDLE & view)
{
	return FALSE;

	MSIDBERROR ret;
	TCHAR szBuff[256];
	DWORD buffSz = sizeof(szBuff) / sizeof(TCHAR);
	ret = MsiViewGetError(view, szBuff, &buffSz);	// MSIDBERROR_NOERROR ret = 1627

	// Not an error.
	if(ret == MSIDBERROR_NOERROR || ret == MSIDBERROR_DUPLICATEKEY)
		return FALSE;

	// Error
	return TRUE;
}







